#include <vtkPoints.h>
#include <vtkPriorityQueue.h>
#include <vtkSmartPointer.h>

void OutputPoints(vtkSmartPointer<vtkPoints> points);
void SortPoints(vtkSmartPointer<vtkPoints> points, int axis);

int main(int, char *[])
{
  // Create a set of points
  vtkSmartPointer<vtkPoints> points =
    vtkSmartPointer<vtkPoints>::New();
  points->InsertNextPoint ( 3.0, 3.0, 3.0 );
  points->InsertNextPoint ( 1.0, 1.0, 1.0 );
  points->InsertNextPoint ( 2.0, 2.0, 2.0 );

  std::cout << "original points: " << std::endl;
  OutputPoints(points);

  int axis = 1;
  SortPoints(points, axis);

  std::cout << "sorted points: " << std::endl;
  OutputPoints(points);

  return EXIT_SUCCESS;
}

void OutputPoints(vtkSmartPointer<vtkPoints> points)
{
  for(vtkIdType ptId = 0; ptId < points->GetNumberOfPoints(); ++ptId)
  {
    double p[3];
    points->GetPoint(ptId,p);
    std::cout << p[0] << " " << p[1] << " " << p[2] << std::endl;
  }
}

void SortPoints(vtkSmartPointer<vtkPoints> points, int axis)
{
  vtkSmartPointer<vtkPriorityQueue> sortedPointIds =
    vtkSmartPointer<vtkPriorityQueue>::New();

  // Put all the points int the priority queue, based on sepecified axis-coord
  vtkIdType numPts = points->GetNumberOfPoints();
  for (vtkIdType ptId = 0; ptId < numPts; ++ptId)
  {
    sortedPointIds->Insert(points->GetPoint(ptId)[axis], ptId);
  }

  vtkSmartPointer<vtkPoints> newPoints =
    vtkSmartPointer<vtkPoints>::New();

  for(vtkIdType ptId = 0; ptId < points->GetNumberOfPoints(); ++ptId)
  {
    double p[3];
    vtkIdType id = sortedPointIds->Pop();
    points->GetPoint(id, p);
    newPoints->InsertNextPoint(p);
  }

  points->ShallowCopy(newPoints);
}
